@@ -66,6 +66,7 @@
 #include "nsContentUtils.h"
 #include "nsDeque.h"
 #include "nsIClassInfoImpl.h"
+#include "nsStringBuffer.h"
 #include "nsThreadUtils.h"
 #include "nsXPCOM.h"
 #include "nsXPCOMCID.h"
@@ -126,6 +127,16 @@ static const char* sPrefsToWatch[] = {
 // The length of time the close handler is allowed to run in milliseconds.
 static PRUint32 gWorkerCloseHandlerTimeoutMS = 10000;
 
+static int sStringFinalizerIndex = -1;
+
+static void
+StringFinalizer(JSContext* aCx,
+                JSString* aStr)
+{
+  NS_ASSERTION(aStr, "Null string!");
+  nsStringBuffer::FromData(JS_GetStringChars(aStr))->Release();
+}
+
 /**
  * Simple class to automatically destroy a JSContext to make error handling
  * easier.
@@ -331,7 +342,7 @@ public:
                    PRBool aClearQueue) {
     NS_ASSERTION(aRunnable, "Null pointer!");
 
-    // No need to enter the monitor because we should already be in it.
+    PR_ASSERT_CURRENT_THREAD_IN_MONITOR(gDOMThreadService->mMonitor);
 
     if (NS_LIKELY(!aTimeoutInterval)) {
       NS_ADDREF(aRunnable);
@@ -352,7 +363,7 @@ public:
     NS_ASSERTION(aTimeoutInterval, "No timeout specified!");
     NS_ASSERTION(aTimeoutInterval!= PR_INTERVAL_NO_TIMEOUT, "Bad timeout!");
 
-    // No need to enter the monitor because we should already be in it.
+    PR_ASSERT_CURRENT_THREAD_IN_MONITOR(gDOMThreadService->mMonitor);
 
     NS_ASSERTION(mWorker->GetExpirationTime() == PR_INTERVAL_NO_TIMEOUT,
                  "Asked to set timeout on a runnable with no close handler!");
@@ -367,6 +378,17 @@ public:
     NS_ASSERTION(!NS_IsMainThread(),
                  "This should *never* run on the main thread!");
 
+    // If the worker has been suspended then we're going to bail here and let
+    // another worker take precedence.
+    if (mWorker->IsSuspended()) {
+      if (gDOMThreadService->QueueSuspendedWorker(this)) {
+        return NS_OK;
+      }
+      // Out of memory maybe? We can suspend the old fashioned way by not
+      // returning here.
+      NS_WARNING("Can't reschedule worker?!");
+    }
+
     // This must have been set up by the thread service
     NS_ASSERTION(gJSContextIndex != BAD_TLS_INDEX, "No context index!");
 
@@ -381,6 +403,11 @@ public:
 
     JS_SetContextPrivate(cx, mWorker);
 
+    // Go ahead and trigger the operation callback for this context before we
+    // try to run any JS. That way we'll be sure to cancel or suspend as soon as
+    // possible if the compilation takes too long.
+    JS_TriggerOperationCallback(cx);
+
     PRBool killWorkerWhenDone;
 
     // Tell the worker which context it will be using
@@ -433,8 +460,6 @@ protected:
   }
 
   void RunQueue(JSContext* aCx, PRBool* aCloseRunnableSet) {
-    PRBool operationCallbackTriggered = PR_FALSE;
-
     while (1) {
       nsCOMPtr<nsIRunnable> runnable;
       {
@@ -469,15 +494,6 @@ protected:
         }
       }
 
-      if (!operationCallbackTriggered) {
-        // Make sure that our operation callback is set to run before starting.
-        // That way we are sure to suspend this worker if needed.
-        JS_TriggerOperationCallback(aCx);
-
-        // Only need to do this the first time.
-        operationCallbackTriggered = PR_TRUE;
-      }
-
       // Clear out any old cruft hanging around in the regexp statics.
       JS_ClearRegExpStatics(aCx);
 
@@ -506,6 +522,7 @@ JSBool
 DOMWorkerOperationCallback(JSContext* aCx)
 {
   nsDOMWorker* worker = (nsDOMWorker*)JS_GetContextPrivate(aCx);
+  NS_ASSERTION(worker, "This must never be null!");
 
   PRBool wasSuspended = PR_FALSE;
   PRBool extraThreadAllowed = PR_FALSE;
@@ -706,6 +723,7 @@ nsDOMThreadService::Init()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(!gDOMThreadService, "Only one instance should ever be created!");
+  NS_ASSERTION(sStringFinalizerIndex == -1, "String finalizer already set!");
 
   nsresult rv;
   nsCOMPtr<nsIObserverService> obs =
@@ -717,6 +735,9 @@ nsDOMThreadService::Init()
 
   obs.forget(&gObserverService);
 
+  sStringFinalizerIndex = JS_AddExternalStringFinalizer(StringFinalizer);
+  NS_ENSURE_TRUE(sStringFinalizerIndex != -1, NS_ERROR_FAILURE);
+
   RegisterPrefCallbacks();
 
   mThreadPool = do_CreateInstance(NS_THREADPOOL_CONTRACTID, &rv);
@@ -827,6 +848,16 @@ nsDOMThreadService::Cleanup()
   // Init fails somehow. We can therefore assume that all services will still
   // be available here.
 
+  {
+    nsAutoMonitor mon(mMonitor);
+
+    NS_ASSERTION(!mPools.Count(), "Live workers left!");
+    mPools.Clear();
+
+    NS_ASSERTION(!mSuspendedWorkers.Length(), "Suspended workers left!");
+    mSuspendedWorkers.Clear();
+  }
+
   if (gObserverService) {
     gObserverService->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
     NS_RELEASE(gObserverService);
@@ -853,11 +884,6 @@ nsDOMThreadService::Cleanup()
   // These must be released after the thread pool is shut down.
   NS_IF_RELEASE(gJSRuntimeService);
   NS_IF_RELEASE(gWorkerSecurityManager);
-
-  nsAutoMonitor mon(mMonitor);
-  NS_ASSERTION(!mPools.Count(), "Live workers left!");
-
-  mPools.Clear();
 }
 
 nsresult
@@ -944,8 +970,7 @@ nsDOMThreadService::SetWorkerTimeout(nsDOMWorker* aWorker,
 void
 nsDOMThreadService::WorkerComplete(nsDOMWorkerRunnable* aRunnable)
 {
-
-  // No need to be in the monitor here because we should already be in it.
+  PR_ASSERT_CURRENT_THREAD_IN_MONITOR(mMonitor);
 
 #ifdef DEBUG
   nsRefPtr<nsDOMWorker>& debugWorker = aRunnable->mWorker;
@@ -959,6 +984,23 @@ nsDOMThreadService::WorkerComplete(nsDOMWorkerRunnable* aRunnable)
   mWorkersInProgress.Remove(aRunnable->mWorker);
 }
 
+PRBool
+nsDOMThreadService::QueueSuspendedWorker(nsDOMWorkerRunnable* aRunnable)
+{
+  nsAutoMonitor mon(mMonitor);
+
+#ifdef DEBUG
+    {
+      // Make sure that the runnable is in mWorkersInProgress.
+      nsRefPtr<nsDOMWorkerRunnable> current;
+      mWorkersInProgress.Get(aRunnable->mWorker, getter_AddRefs(current));
+      NS_ASSERTION(current == aRunnable, "Something crazy wrong here!");
+    }
+#endif
+
+  return mSuspendedWorkers.AppendElement(aRunnable) ? PR_TRUE : PR_FALSE;
+}
+
 /* static */
 JSContext*
 nsDOMThreadService::CreateJSContext()
@@ -981,8 +1023,7 @@ nsDOMThreadService::CreateJSContext()
   };
   JS_SetContextSecurityCallbacks(cx, &securityCallbacks);
 
-  static JSDebugHooks debugHooks;
-  JS_SetContextDebugHooks(cx, &debugHooks);
+  JS_ClearContextDebugHooks(cx);
 
   nsresult rv = nsContentUtils::XPConnect()->
     SetSecurityManagerForJSContext(cx, gWorkerSecurityManager, 0);
@@ -1034,7 +1075,7 @@ nsDOMThreadService::GetPoolForGlobal(nsIScriptGlobalObject* aGlobalObject,
 void
 nsDOMThreadService::TriggerOperationCallbackForPool(nsDOMWorkerPool* aPool)
 {
-  nsAutoMonitor mon(mMonitor);
+  PR_ASSERT_CURRENT_THREAD_IN_MONITOR(mMonitor);
 
   // See if we need to trigger the operation callback on any currently running
   // contexts.
@@ -1048,6 +1089,46 @@ nsDOMThreadService::TriggerOperationCallbackForPool(nsDOMWorkerPool* aPool)
   }
 }
 
+void
+nsDOMThreadService::RescheduleSuspendedWorkerForPool(nsDOMWorkerPool* aPool)
+{
+  PR_ASSERT_CURRENT_THREAD_IN_MONITOR(mMonitor);
+
+  PRUint32 count = mSuspendedWorkers.Length();
+  if (!count) {
+    // Nothing to do here.
+    return;
+  }
+
+  nsTArray<nsDOMWorkerRunnable*> others(count);
+
+  for (PRUint32 index = 0; index < count; index++) {
+    nsDOMWorkerRunnable* runnable = mSuspendedWorkers[index];
+
+#ifdef DEBUG
+    {
+      // Make sure that the runnable never left mWorkersInProgress.
+      nsRefPtr<nsDOMWorkerRunnable> current;
+      mWorkersInProgress.Get(runnable->mWorker, getter_AddRefs(current));
+      NS_ASSERTION(current == runnable, "Something crazy wrong here!");
+    }
+#endif
+
+    if (runnable->mWorker->Pool() == aPool) {
+#ifdef DEBUG
+      nsresult rv =
+#endif
+      mThreadPool->Dispatch(runnable, NS_DISPATCH_NORMAL);
+      NS_ASSERTION(NS_SUCCEEDED(rv), "This shouldn't ever fail!");
+    }
+    else {
+      others.AppendElement(runnable);
+    }
+  }
+
+  mSuspendedWorkers.SwapElements(others);
+}
+
 void
 nsDOMThreadService::CancelWorkersForGlobal(nsIScriptGlobalObject* aGlobalObject)
 {
@@ -1056,7 +1137,11 @@ nsDOMThreadService::CancelWorkersForGlobal(nsIScriptGlobalObject* aGlobalObject)
   nsRefPtr<nsDOMWorkerPool> pool = GetPoolForGlobal(aGlobalObject, PR_TRUE);
   if (pool) {
     pool->Cancel();
+
+    nsAutoMonitor mon(mMonitor);
+
     TriggerOperationCallbackForPool(pool);
+    RescheduleSuspendedWorkerForPool(pool);
   }
 }
 
@@ -1068,6 +1153,8 @@ nsDOMThreadService::SuspendWorkersForGlobal(nsIScriptGlobalObject* aGlobalObject
   nsRefPtr<nsDOMWorkerPool> pool = GetPoolForGlobal(aGlobalObject, PR_FALSE);
   if (pool) {
     pool->Suspend();
+
+    nsAutoMonitor mon(mMonitor);
     TriggerOperationCallbackForPool(pool);
   }
 }
@@ -1080,7 +1167,11 @@ nsDOMThreadService::ResumeWorkersForGlobal(nsIScriptGlobalObject* aGlobalObject)
   nsRefPtr<nsDOMWorkerPool> pool = GetPoolForGlobal(aGlobalObject, PR_FALSE);
   if (pool) {
     pool->Resume();
+
+    nsAutoMonitor mon(mMonitor);
+
     TriggerOperationCallbackForPool(pool);
+    RescheduleSuspendedWorkerForPool(pool);
   }
 }
 
@@ -1140,24 +1231,59 @@ nsDOMThreadService::ChangeThreadPoolMaxThreads(PRInt16 aDelta)
   return NS_OK;
 }
 
+// static
 nsIJSRuntimeService*
 nsDOMThreadService::JSRuntimeService()
 {
   return gJSRuntimeService;
 }
 
+// static
 nsIThreadJSContextStack*
 nsDOMThreadService::ThreadJSContextStack()
 {
   return gThreadJSContextStack;
 }
 
+// static
 nsIXPCSecurityManager*
 nsDOMThreadService::WorkerSecurityManager()
 {
   return gWorkerSecurityManager;
 }
 
+// static
+jsval
+nsDOMThreadService::ShareStringAsJSVal(JSContext* aCx,
+                                       const nsAString& aString)
+{
+  NS_ASSERTION(sStringFinalizerIndex != -1, "Bad index!");
+  NS_ASSERTION(aCx, "Null context!");
+
+  PRUint32 length = aString.Length();
+  if (!length) {
+    JSAtom* atom = aCx->runtime->atomState.emptyAtom;
+    return ATOM_KEY(atom);
+  }
+
+  nsStringBuffer* buf = nsStringBuffer::FromString(aString);
+  if (!buf) {
+    NS_WARNING("Can't share this string buffer!");
+    return JSVAL_VOID;
+  }
+
+  JSString* str =
+    JS_NewExternalString(aCx, reinterpret_cast<jschar*>(buf->Data()), length,
+                         sStringFinalizerIndex);
+  if (str) {
+    buf->AddRef();
+    return STRING_TO_JSVAL(str);
+  }
+
+  NS_WARNING("JS_NewExternalString failed!");
+  return JSVAL_VOID;
+}
+
 /**
  * See nsIEventTarget
  */
