@@ -16,10 +16,11 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Igor Bukanov  <igor@mir2.org>
+ *                 Bob Clary <bob@bclary.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,32 +36,46 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-416737-02.js';
+var gTestfile = 'regress-280769-4.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 416737;
-var summary = 'Do not assert: *pc == JSOP_GETARG';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 280769;
+var summary = 'Do not overflow 64K length of char sequence in RegExp []';
+var actual = 'No Crash';
+var expect = 'No Crash';
+
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+var N = 20 * 1000; // It should be that N*6  > 64K and N < 32K
 
-function test()
+var prefixes = ["000", "00", "0"];
+
+function to_4_hex(i)
 {
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
-
-  var f = function([]){ function n(){} };
-  if (typeof dis == 'function')
-  {
-    dis(f);
+  var printed = i.toString(16);
+  if (printed.length < 4) {
+    printed= prefixes[printed.length - 1]+printed;
   }
-  print(f);
+  return printed;
 
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
 }
+
+var a = new Array(N);
+for (var i = 0; i != N; ++i) {
+  a[i] = to_4_hex(2*i);
+}
+
+var str = '[\\u'+a.join('\\u')+']';
+// str is [\u0000\u0002\u0004...\u<printed value of 2N>]
+
+var re = new RegExp(str);
+
+var string_to_match = String.fromCharCode(2 * (N - 1));
+
+var value = re.exec(string_to_match);
+
+var expect =  string_to_match;
+var actual = value ? value[0] : value;
+
+reportCompare(expect, actual, summary);
