@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,48 +35,26 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-354499-02.js';
+var gTestfile = 'regress-469405-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 354499;
-var summary = 'Iterating over Array elements';
+var BUGNUMBER = 469405;
+var summary = 'Do not assert: !JSVAL_IS_PRIMITIVE(regs.sp[-2])';
 var actual = '';
 var expect = '';
 
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
-
-function test()
-{
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
-
-  expect = actual = 'No Crash';
-
-  function get_value()
-  {
-    // Unroot the first element
-    this[0] = 0;
-
-    // Call gc to collect atom corresponding to Math.sqrt(2)
-    gc();
-  }
-
-  var counter = 2;
-  Iterator.prototype.next = function()
-    {
-      if (counter-- <= 0) throw StopIteration;
-      var a = [Math.sqrt(2), 1];
-      a.__defineGetter__(1, get_value);
-      return a;
-    };
-
-  for (i in [1])
-    ;
-
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
+try
+{ 
+  (function() {
+    var a, b;
+    for each (a in [{}, {__iterator__: function(){}}]) for (b in a) { }
+  })();
 }
+catch(ex)
+{
+  print('caught ' + ex);
+}
+
+reportCompare(expect, actual, summary);
