@@ -82,7 +82,6 @@ enum {
   eCommand_ID_Last  = 4
 };
 
-
 NS_IMPL_ISUPPORTS1(nsNativeMenuServiceX, nsINativeMenuService)
 
 NS_IMETHODIMP nsNativeMenuServiceX::CreateNativeMenuBar(nsIWidget* aParent, nsIContent* aMenuBarNode)
@@ -96,7 +95,6 @@ NS_IMETHODIMP nsNativeMenuServiceX::CreateNativeMenuBar(nsIWidget* aParent, nsIC
   return mb->Create(aParent, aMenuBarNode);
 }
 
-
 nsMenuBarX::nsMenuBarX()
 : mParentWindow(nsnull),
   mCurrentCommandID(eCommand_ID_Last),
@@ -109,7 +107,6 @@ nsMenuBarX::nsMenuBarX()
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-
 nsMenuBarX::~nsMenuBarX()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
@@ -141,7 +138,6 @@ nsMenuBarX::~nsMenuBarX()
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-
 nsresult nsMenuBarX::Create(nsIWidget* aParent, nsIContent* aContent)
 {
   if (!aParent || !aContent)
@@ -161,9 +157,10 @@ nsresult nsMenuBarX::Create(nsIWidget* aParent, nsIContent* aContent)
   ConstructNativeMenus();
 
   // Give this to the parent window. The parent takes ownership.
-  return mParentWindow->SetMenuBar(this);
-}
+  static_cast<nsCocoaWindow*>(mParentWindow)->SetMenuBar(this);
 
+  return NS_OK;
+}
 
 void nsMenuBarX::ConstructNativeMenus()
 {
@@ -185,13 +182,11 @@ void nsMenuBarX::ConstructNativeMenus()
   }  
 }
 
-
 PRUint32 nsMenuBarX::GetMenuCount()
 {
   return mMenuArray.Length();
 }
 
-
 bool nsMenuBarX::MenuContainsAppMenu()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
@@ -202,7 +197,6 @@ bool nsMenuBarX::MenuContainsAppMenu()
   NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(false);
 }
 
-
 nsresult nsMenuBarX::InsertMenuAtIndex(nsMenuX* aMenu, PRUint32 aIndex)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
@@ -237,7 +231,6 @@ nsresult nsMenuBarX::InsertMenuAtIndex(nsMenuX* aMenu, PRUint32 aIndex)
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
 }
 
-
 void nsMenuBarX::RemoveMenuAtIndex(PRUint32 aIndex)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
@@ -257,7 +250,6 @@ void nsMenuBarX::RemoveMenuAtIndex(PRUint32 aIndex)
   NS_OBJC_END_TRY_ABORT_BLOCK;
 }
 
-
 void nsMenuBarX::ForceUpdateNativeMenuAt(const nsAString& indexString)
 {
   NSString* locationString = [NSString stringWithCharacters:indexString.BeginReading() length:indexString.Length()];
@@ -318,7 +310,6 @@ void nsMenuBarX::ForceUpdateNativeMenuAt(const nsAString& indexString)
   }
 }
 
-
 // Calling this forces a full reload of the menu system, reloading all native
 // menus and their items.
 // Without this testing is hard because changes to the DOM affect the native
@@ -333,7 +324,6 @@ void nsMenuBarX::ForceNativeMenuReload()
   ConstructNativeMenus();
 }
 
-
 nsMenuX* nsMenuBarX::GetMenuAt(PRUint32 aIndex)
 {
   if (mMenuArray.Length() <= aIndex) {
@@ -343,7 +333,6 @@ nsMenuX* nsMenuBarX::GetMenuAt(PRUint32 aIndex)
   return mMenuArray[aIndex];
 }
 
-
 nsresult nsMenuBarX::Paint()
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
@@ -373,6 +362,44 @@ nsresult nsMenuBarX::Paint()
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
 }
 
+// Returns the 'key' attribute of the 'shortcutID' object (if any) in the
+// currently active menubar's DOM document.  'shortcutID' should be the id
+// (i.e. the name) of a component that defines a commonly used (and
+// localized) cmd+key shortcut, and belongs to a keyset containing similar
+// objects.  For example "key_selectAll".  Returns a value that can be
+// compared to the first character of [NSEvent charactersIgnoringModifiers]
+// when [NSEvent modifierFlags] == NSCommandKeyMask.
+char nsMenuBarX::GetLocalizedAccelKey(char *shortcutID)
+{
+  if (!sLastGeckoMenuBarPainted)
+    return 0;
+
+  nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(sLastGeckoMenuBarPainted->mDocument));
+  if (!domDoc)
+    return 0;
+
+  NS_ConvertASCIItoUTF16 shortcutIDStr((const char *)shortcutID);
+  nsCOMPtr<nsIDOMElement> shortcutElement;
+  domDoc->GetElementById(shortcutIDStr, getter_AddRefs(shortcutElement));
+  nsCOMPtr<nsIContent> shortcutContent = do_QueryInterface(shortcutElement);
+  if (!shortcutContent)
+    return 0;
+
+  nsAutoString key;
+  shortcutContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::key, key);
+  NS_LossyConvertUTF16toASCII keyASC(key.get());
+  const char *keyASCPtr = keyASC.get();
+  if (!keyASCPtr)
+    return 0;
+  // If keyID's 'key' attribute isn't exactly one character long, it's not
+  // what we're looking for.
+  if (strlen(keyASCPtr) != sizeof(char))
+    return 0;
+  // Make sure retval is lower case.
+  char retval = tolower(keyASCPtr[0]);
+
+  return retval;
+}
 
 // Hide the item in the menu by setting the 'hidden' attribute. Returns it in |outHiddenNode| so
 // the caller can hang onto it if they so choose. It is acceptable to pass nsull
@@ -391,7 +418,6 @@ void nsMenuBarX::HideItem(nsIDOMDocument* inDoc, const nsAString & inID, nsICont
   }
 }
 
-
 // Do what is necessary to conform to the Aqua guidelines for menus.
 void nsMenuBarX::AquifyMenuBar()
 {
@@ -424,7 +450,6 @@ void nsMenuBarX::AquifyMenuBar()
   }
 }
 
-
 // for creating menu items destined for the Application menu
 NSMenuItem* nsMenuBarX::CreateNativeAppMenuItem(nsMenuX* inMenu, const nsAString& nodeID, SEL action,
                                                 int tag, NativeMenuItemTarget* target)
@@ -496,7 +521,6 @@ NSMenuItem* nsMenuBarX::CreateNativeAppMenuItem(nsMenuX* inMenu, const nsAString
   NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
 }
 
-
 // build the Application menu shared by all menu bars
 nsresult nsMenuBarX::CreateApplicationMenu(nsMenuX* inMenu)
 {
@@ -650,32 +674,27 @@ nsresult nsMenuBarX::CreateApplicationMenu(nsMenuX* inMenu)
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
 }
 
-
 void nsMenuBarX::SetParent(nsIWidget* aParent)
 {
   mParentWindow = aParent;
 }
 
-
 //
 // nsIMutationObserver
 //
 
-
 void nsMenuBarX::CharacterDataWillChange(nsIDocument* aDocument,
                                          nsIContent* aContent,
                                          CharacterDataChangeInfo* aInfo)
 {
 }
 
-
 void nsMenuBarX::CharacterDataChanged(nsIDocument* aDocument,
                                       nsIContent* aContent,
                                       CharacterDataChangeInfo* aInfo)
 {
 }
 
-
 void nsMenuBarX::ContentAppended(nsIDocument* aDocument, nsIContent* aContainer,
                                  PRInt32 aNewIndexInContainer)
 {
@@ -687,13 +706,17 @@ void nsMenuBarX::ContentAppended(nsIDocument* aDocument, nsIContent* aContainer,
   }
 }
 
-
 void nsMenuBarX::NodeWillBeDestroyed(const nsINode * aNode)
 {
   // our menu bar node is being destroyed
   mDocument = nsnull;
 }
 
+void nsMenuBarX::AttributeWillChange(nsIDocument* aDocument,
+                                     nsIContent* aContent, PRInt32 aNameSpaceID,
+                                     nsIAtom* aAttribute, PRInt32 aModType)
+{
+}
 
 void nsMenuBarX::AttributeChanged(nsIDocument * aDocument, nsIContent * aContent,
                                   PRInt32 aNameSpaceID, nsIAtom * aAttribute,
@@ -704,7 +727,6 @@ void nsMenuBarX::AttributeChanged(nsIDocument * aDocument, nsIContent * aContent
     obs->ObserveAttributeChanged(aDocument, aContent, aAttribute);
 }
 
-
 void nsMenuBarX::ContentRemoved(nsIDocument * aDocument, nsIContent * aContainer,
                                 nsIContent * aChild, PRInt32 aIndexInContainer)
 {
@@ -730,7 +752,6 @@ void nsMenuBarX::ContentRemoved(nsIDocument * aDocument, nsIContent * aContainer
   }
 }
 
-
 void nsMenuBarX::ContentInserted(nsIDocument * aDocument, nsIContent * aContainer,
                                  nsIContent * aChild, PRInt32 aIndexInContainer)
 {
@@ -762,12 +783,10 @@ void nsMenuBarX::ContentInserted(nsIDocument * aDocument, nsIContent * aContaine
   }
 }
 
-
 void nsMenuBarX::ParentChainChanged(nsIContent *aContent)
 {
 }
 
-
 // For change management, we don't use a |nsSupportsHashtable| because we know that the
 // lifetime of all these items is bounded by the lifetime of the menubar. No need to add
 // any more strong refs to the picture because the containment hierarchy already uses
@@ -778,21 +797,18 @@ void nsMenuBarX::RegisterForContentChanges(nsIContent *aContent, nsChangeObserve
   mObserverTable.Put(&key, aMenuObject);
 }
 
-
 void nsMenuBarX::UnregisterForContentChanges(nsIContent *aContent)
 {
   nsVoidKey key(aContent);
   mObserverTable.Remove(&key);
 }
 
-
 nsChangeObserver* nsMenuBarX::LookupContentChangeObserver(nsIContent* aContent)
 {
   nsVoidKey key(aContent);
   return reinterpret_cast<nsChangeObserver*>(mObserverTable.Get(&key));
 }
 
-
 // Given a menu item, creates a unique 4-character command ID and
 // maps it to the item. Returns the id for use by the client.
 PRUint32 nsMenuBarX::RegisterForCommand(nsMenuItemX* inMenuItem)
@@ -812,7 +828,6 @@ PRUint32 nsMenuBarX::RegisterForCommand(nsMenuItemX* inMenuItem)
   return mCurrentCommandID;
 }
 
-
 // Removes the mapping between the given 4-character command ID
 // and its associated menu item.
 void nsMenuBarX::UnregisterCommand(PRUint32 inCommandID)
@@ -821,21 +836,18 @@ void nsMenuBarX::UnregisterCommand(PRUint32 inCommandID)
   mObserverTable.Remove(&key);
 }
 
-
 nsMenuItemX* nsMenuBarX::GetMenuItemForCommandID(PRUint32 inCommandID)
 {
   nsPRUint32Key key(inCommandID);
   return reinterpret_cast<nsMenuItemX*>(mObserverTable.Get(&key));
 }
 
-
 //
 // Objective-C class used to allow us to have keyboard commands
 // look like they are doing something but actually do nothing.
 // We allow mouse actions to work normally.
 //
 
-
 // This tells us whether or not pKE is on the stack from a GeckoNSMenu. If it
 // is nil, it is not on the stack. The non-nil value is the object that put it
 // on the stack first.
@@ -868,7 +880,6 @@ static BOOL gActOnSpecialCommands = YES;
   NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(NO);
 }
 
-
 -(void)actOnKeyEquivalent:(NSEvent *)theEvent
 {
   gActOnKeyEquiv = YES;
@@ -876,7 +887,6 @@ static BOOL gActOnSpecialCommands = YES;
   gActOnKeyEquiv = NO;
 }
 
-
 - (void)performMenuUserInterfaceEffectsForEvent:(NSEvent*)theEvent
 {
   gActOnSpecialCommands = NO;
@@ -886,12 +896,10 @@ static BOOL gActOnSpecialCommands = YES;
 
 @end
 
-
 //
 // Objective-C class used as action target for menu items
 //
 
-
 @implementation NativeMenuItemTarget
 
 // called when some menu item in this menu gets hit
